(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators'), require('tslib')) :
	typeof define === 'function' && define.amd ? define(['exports', 'rxjs', 'rxjs/operators', 'tslib'], factory) :
	(factory((global.rxjs_extra = global.rxjs_extra || {}),global.rxjs,global.rxjs_operators,global.tslib_1));
}(this, (function (exports,rxjs,rxjs_operators,tslib_1) { 'use strict';

var presetTimer = function (delays, scheduler) {
    return rxjs.range(delays.length).pipe(rxjs_operators.concatMap(function (index) { return rxjs.of(index).pipe(rxjs_operators.delay(delays[index], scheduler)); }));
};

var randomNumberGenerator = function (min, max) { return Math.random() * (max - min) + min; };

var randomTimer = function (min, max, generator, scheduler) {
    if (generator === void 0) { generator = randomNumberGenerator; }
    if (scheduler === void 0) { scheduler = rxjs.asyncScheduler; }
    return rxjs.defer(function () {
        var index = 0;
        return rxjs.defer(function () { return rxjs.of(index++); }).pipe(rxjs_operators.concatMap(function (value) { return rxjs.of(value).pipe(rxjs_operators.delay(generator(min, max), scheduler)); }), rxjs_operators.take(1), rxjs_operators.repeat());
    });
};

var schedulerNow = function (scheduler) { return (scheduler || rxjs.queueScheduler).now(); };

var DebugObserver = (function (_super) {
    tslib_1.__extends(DebugObserver, _super);
    function DebugObserver(identifier, logger, scheduler) {
        if (logger === void 0) { logger = console.debug; }
        var _this = _super.call(this, function (v) { return _this.log('N', v); }, function (e) { return _this.log('E', e); }, function () { return _this.logComplete('C'); }) || this;
        _this.identifier = identifier;
        _this.logger = logger;
        _this.scheduler = scheduler;
        _this.start = schedulerNow(_this.scheduler);
        return _this;
    }
    Object.defineProperty(DebugObserver.prototype, "timeSinceStart", {
        get: function () {
            return schedulerNow(this.scheduler) - this.start;
        },
        enumerable: true,
        configurable: true
    });
    DebugObserver.prototype.messageTag = function (notificationType) {
        return "[" + this.identifier + "]" + notificationType + "@" + padWithZeroes(this.timeSinceStart, 7) + ":";
    };
    DebugObserver.prototype.log = function (notificationType, value) {
        this.logger(this.messageTag(notificationType), value);
    };
    DebugObserver.prototype.logComplete = function (notificationType) {
        this.logger(this.messageTag(notificationType));
    };
    return DebugObserver;
}(rxjs.Subscriber));
var padWithZeroes = function (number, length) {
    var myString = number.toString();
    while (myString.length < length) {
        myString = ' ' + myString;
    }
    return myString;
};

var LocalStoragePersistentStorage = (function () {
    function LocalStoragePersistentStorage() {
    }
    LocalStoragePersistentStorage.prototype.setItem = function (key, value) {
        localStorage.setItem(key, JSON.stringify(value));
        return true;
    };
    LocalStoragePersistentStorage.prototype.getItem = function (key) {
        var stored = localStorage.getItem(key);
        return stored ? JSON.parse(stored) : null;
    };
    return LocalStoragePersistentStorage;
}());
var localPersistentStorage = new LocalStoragePersistentStorage();

var PersistentSubject = (function (_super) {
    tslib_1.__extends(PersistentSubject, _super);
    function PersistentSubject(storageKey, defaultValue, storage) {
        if (storage === void 0) { storage = localPersistentStorage; }
        var _this = this;
        var stored = storage.getItem(storageKey);
        var value = stored ? stored : defaultValue;
        _this = _super.call(this, value) || this;
        _this.storage = storage;
        _this.storageKey = storageKey;
        return _this;
    }
    PersistentSubject.prototype.next = function (value) {
        this.storage.setItem(this.storageKey, value);
        _super.prototype.next.call(this, value);
    };
    return PersistentSubject;
}(rxjs.BehaviorSubject));

var CacheMode;
(function (CacheMode) {
    CacheMode[CacheMode["Default"] = 0] = "Default";
    CacheMode[CacheMode["TolerateExpired"] = 1] = "TolerateExpired";
    CacheMode[CacheMode["SilentRefresh"] = 2] = "SilentRefresh";
})(CacheMode || (CacheMode = {}));
function cache(windowTime, mode, scheduler) {
    if (mode === void 0) { mode = CacheMode.Default; }
    return function (source) {
        if (mode === CacheMode.SilentRefresh) {
            return source.pipe(rxjs_operators.timestamp(scheduler), rxjs_operators.publishReplay(1, Number.POSITIVE_INFINITY, scheduler), rxjs_operators.refCount(), rxjs_operators.takeWhile(function (item, i) {
                if (i === 0) {
                    return schedulerNow(scheduler) > item.timestamp + windowTime;
                }
                return false;
            }, true), rxjs_operators.filter(function (_, i) { return i === 0; }), rxjs_operators.map(function (item) { return item.value; }));
        }
        else if (mode === CacheMode.TolerateExpired) {
            return source.pipe(rxjs_operators.timestamp(scheduler), rxjs_operators.publishReplay(1, Number.POSITIVE_INFINITY, scheduler), rxjs_operators.refCount(), rxjs_operators.takeWhile(function (item) { return schedulerNow(scheduler) > item.timestamp + windowTime; }, true), rxjs_operators.map(function (item) { return item.value; }));
        }
        else {
            return source.pipe(rxjs_operators.publishReplay(1, windowTime, scheduler), rxjs_operators.refCount(), rxjs_operators.take(1));
        }
    };
}

var delayComplete = function (time, scheduler) {
    return rxjs_operators.multicast(new rxjs.Subject(), function (s) { return rxjs.concat(s, rxjs.of(void 0).pipe(rxjs_operators.delay(time, scheduler), rxjs_operators.ignoreElements())); });
};

var errorWhen = function (predicate, errorOrFactory) {
    if (errorOrFactory === void 0) { errorOrFactory = new Error(); }
    return rxjs_operators.map(function (value) {
        if (predicate(value)) {
            throw typeof errorOrFactory === 'function' ? errorOrFactory(value) : errorOrFactory;
        }
        return value;
    });
};

var FinalizeReason;
(function (FinalizeReason) {
    FinalizeReason["Unsubscribe"] = "unsubscribe";
    FinalizeReason["Complete"] = "complete";
    FinalizeReason["Error"] = "error";
})(FinalizeReason || (FinalizeReason = {}));
var finalizeWithReason = function (callback) {
    return function (source) {
        return rxjs.defer(function () {
            var completed = false;
            var errored = false;
            return source.pipe(rxjs_operators.tap({
                error: function () { return errored = true; },
                complete: function () { return completed = true; },
            }), rxjs_operators.finalize(function () {
                if (errored) {
                    callback(FinalizeReason.Error);
                }
                else if (completed) {
                    callback(FinalizeReason.Complete);
                }
                else {
                    callback(FinalizeReason.Unsubscribe);
                }
            }));
        });
    };
};

var queueTime = function (timeDelay, scheduler) {
    return function (source) { return rxjs.defer(function () {
        var nextEarliestEmission = null;
        return source.pipe(rxjs_operators.concatMap(function (value) {
            var now = schedulerNow(scheduler);
            var expectedEmissionTime = nextEarliestEmission;
            if (expectedEmissionTime === null || now > expectedEmissionTime) {
                nextEarliestEmission = now + timeDelay;
                return rxjs.of(value);
            }
            else {
                var requiredDelay = expectedEmissionTime - now;
                nextEarliestEmission = now + timeDelay + requiredDelay;
                return rxjs.of(value).pipe(rxjs_operators.delay(requiredDelay, scheduler));
            }
        }));
    }); };
};

var takeUntilComplete = function (notifier) {
    return rxjs_operators.takeUntil(rxjs.concat(notifier.pipe(rxjs_operators.ignoreElements()), rxjs.of(undefined)));
};

var tapSubscribe = function (callback) {
    return function (source) {
        return rxjs.defer(function () {
            callback();
            return source;
        });
    };
};

var randomDelay = function (min, max, generator, scheduler) {
    if (generator === void 0) { generator = randomNumberGenerator; }
    return rxjs_operators.concatMap(function (value) { return rxjs.of(value).pipe(rxjs_operators.delay(generator(min, max), scheduler)); });
};

var retryTime = function (delayTime, scheduler) {
    return rxjs_operators.retryWhen(function (errors) { return typeof delayTime === 'number'
        ? errors.pipe(rxjs_operators.delay(delayTime, scheduler))
        : errors.pipe(rxjs_operators.concatMap(function (e, index) { return delayTime.length === index
            ? rxjs.throwError(e)
            : rxjs.of(e).pipe(rxjs_operators.delay(delayTime[index], scheduler)); })); });
};



var _operators = Object.freeze({
	get CacheMode () { return CacheMode; },
	cache: cache,
	delayComplete: delayComplete,
	errorWhen: errorWhen,
	get FinalizeReason () { return FinalizeReason; },
	finalizeWithReason: finalizeWithReason,
	queueTime: queueTime,
	takeUntilComplete: takeUntilComplete,
	tapSubscribe: tapSubscribe,
	randomDelay: randomDelay,
	retryTime: retryTime
});

var operators = _operators;

exports.operators = operators;
exports.presetTimer = presetTimer;
exports.randomTimer = randomTimer;
exports.DebugObserver = DebugObserver;
exports.PersistentSubject = PersistentSubject;
exports.randomNumberGenerator = randomNumberGenerator;
exports.LocalStoragePersistentStorage = LocalStoragePersistentStorage;
exports.localPersistentStorage = localPersistentStorage;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=rxjs-extra.umd.js.map
