"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var operators_1 = require("rxjs/operators");
var now_1 = require("../utils/now");
var CacheMode;
(function (CacheMode) {
    CacheMode[CacheMode["Default"] = 0] = "Default";
    CacheMode[CacheMode["TolerateExpired"] = 1] = "TolerateExpired";
    CacheMode[CacheMode["SilentRefresh"] = 2] = "SilentRefresh";
})(CacheMode = exports.CacheMode || (exports.CacheMode = {}));
function cache(windowTime, mode, scheduler) {
    if (mode === void 0) { mode = CacheMode.Default; }
    return function (source) {
        if (mode === CacheMode.SilentRefresh) {
            return source.pipe(operators_1.timestamp(scheduler), operators_1.publishReplay(1, Number.POSITIVE_INFINITY, scheduler), operators_1.refCount(), operators_1.takeWhile(function (item, i) {
                if (i === 0) {
                    return now_1.schedulerNow(scheduler) > item.timestamp + windowTime;
                }
                return false;
            }, true), operators_1.filter(function (_, i) { return i === 0; }), operators_1.map(function (item) { return item.value; }));
        }
        else if (mode === CacheMode.TolerateExpired) {
            return source.pipe(operators_1.timestamp(scheduler), operators_1.publishReplay(1, Number.POSITIVE_INFINITY, scheduler), operators_1.refCount(), operators_1.takeWhile(function (item) { return now_1.schedulerNow(scheduler) > item.timestamp + windowTime; }, true), operators_1.map(function (item) { return item.value; }));
        }
        else {
            return source.pipe(operators_1.publishReplay(1, windowTime, scheduler), operators_1.refCount(), operators_1.take(1));
        }
    };
}
exports.cache = cache;
//# sourceMappingURL=cache.js.map