"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var FinalizeReason;
(function (FinalizeReason) {
    FinalizeReason["Unsubscribe"] = "unsubscribe";
    FinalizeReason["Complete"] = "complete";
    FinalizeReason["Error"] = "error";
})(FinalizeReason = exports.FinalizeReason || (exports.FinalizeReason = {}));
exports.finalizeWithReason = function (callback) {
    return function (source) {
        return rxjs_1.defer(function () {
            var completed = false;
            var errored = false;
            return source.pipe(operators_1.tap({
                error: function () { return errored = true; },
                complete: function () { return completed = true; },
            }), operators_1.finalize(function () {
                if (errored) {
                    callback(FinalizeReason.Error);
                }
                else if (completed) {
                    callback(FinalizeReason.Complete);
                }
                else {
                    callback(FinalizeReason.Unsubscribe);
                }
            }));
        });
    };
};
//# sourceMappingURL=finalizeWithReason.js.map